from typing import List, Tuple
from argparse import ArgumentParser
import numpy as np
import pylab
from utils import data_utils


def plot_wave(timepoints: List[float], wave: np.array, wave_name: str, dim1: int, dim2: int,
              plot_num: int, plt_color: str):
    pylab.subplot(dim1, dim2, plot_num)
    pylab.plot(timepoints, wave, color=plt_color)
    pylab.xlim([np.min(timepoints), np.max(timepoints)])
    # pylab.ylim([-amplitude, amplitude])

    pylab.title(wave_name)

    plot_num += 1

    print("%s: %f" % (wave_name, wave.max()))

    return plot_num


def plot_waves(tps: List[float], data, plot_dim1: int, plot_dim2: int, plot_num: int, plt_color: str):
    for i in range(data["waves"].shape[0]):
        plot_num = plot_wave(tps, data["waves"][i], data["wave_types"][i], plot_dim1, plot_dim2, plot_num, plt_color)

    return plot_num


def main():
    parser: ArgumentParser = ArgumentParser()
    parser.add_argument('--timepoints', type=str, default="", help="")
    parser.add_argument('--per_range', type=str, default="24.0,24.0", help="")
    parser.add_argument('--phase_range', type=str, default="-6.28,6.28", help="")
    parser.add_argument('--amp_range', type=str, default="1,1", help="")
    parser.add_argument('--slope_range', type=str, default="0.1,0.1", help="")
    parser.add_argument('--offset_range', type=str, default="0,0", help="")
    parser.add_argument('--snr_range', type=str, default="1,10", help="")
    parser.add_argument('--outlier_prob', type=float, default=0.0, help="")
    args = parser.parse_args()

    if len(args.timepoints) == 0:
        tps: List[float] = list(np.arange(0, 48, 1))
    else:
        tps: List[float] = [float(x) for x in args.timepoints.split(",")]

    plot_dim1: int = 4
    plot_dim2: int = 3
    plot_num: int = 1

    per_range: Tuple[float, float] = (float(args.per_range.split(",")[0]), float(args.per_range.split(",")[1]))
    phase_range: Tuple[float, float] = (float(args.phase_range.split(",")[0]), float(args.phase_range.split(",")[1]))
    slope_range: Tuple[float, float] = (float(args.slope_range.split(",")[0]), float(args.slope_range.split(",")[1]))
    amp_range: Tuple[float, float] = (float(args.amp_range.split(",")[0]), float(args.amp_range.split(",")[1]))
    offset_range: Tuple[float, float] = (float(args.offset_range.split(",")[0]),
                                         float(args.offset_range.split(",")[1]))
    snr_range: Tuple[float, float] = (float(args.snr_range.split(",")[0]), float(args.snr_range.split(",")[1]))

    print(per_range, slope_range, amp_range, offset_range, args.outlier_prob)

    for per_type in ["periodic", "aperiodic"]:
        if per_type == "periodic":
            num: int = 7
            plt_color: str = "green"
        else:
            num: int = 5
            plt_color: str = "red"

        data = data_utils.get_data(tps, num, per_type, period_range=per_range, phase_range=phase_range,
                                   slope_range=slope_range, amp_range=amp_range, offset_range=offset_range,
                                   snr_range=snr_range, outlier_prob=args.outlier_prob)
        plot_num = plot_waves(tps, data, plot_dim1, plot_dim2, plot_num, plt_color)

    pylab.tight_layout()

    # pylab.show()

    pylab.savefig("sampleTraj.eps")


if __name__ == "__main__":
    main()
